#ifndef __ATARI__

#include <allegro.h>
#include <stdio.h>
#include "game\all_definitions.h"

#define SHOW_AREAS 1

//////////////////////////////////////////////////////////////////////////
extern struct s_object g_objects[OBJECT_LAST_OBJECT];
extern struct s_picture g_pictures[PICTURE_LAST_PICTURE];
extern struct s_palette g_palettes[PALETTE_LAST_PALETTE];
extern struct s_room g_rooms[ROOM_LAST_ROOM];
extern struct s_area g_areas[AREA_LAST_AREA];
extern struct s_container g_containers[CONTAINER_LAST_CONTAINER];
//////////////////////////////////////////////////////////////////////////

BITMAP *mouse_sprite_normal;
BITMAP *mouse_sprite_waiting;

BITMAP *screen_buffer;
BITMAP *console_buffer;
size_t console_cursor;


void redraw_screen(void)
{
	stretch_blit(screen_buffer,screen,0,0,screen_buffer->w,screen_buffer->h,0,0,screen_buffer->w*4,screen_buffer->h*2);
}

void redraw_console(void)
{
	stretch_blit(console_buffer,screen,0,0,console_buffer->w,console_buffer->h,0,320,console_buffer->w*2,console_buffer->h*2);
}


void *memory_manager_alloc(word size)
{
	return malloc(size);
}

// display container elements on the screen
// can be in gfx form or as text
// In demo we show as graphics
object_id ui_container_choose_item(container_id container)
{
	object_id result=OBJECT_NONE;
	object_id obj;
	picture_id pic;
	byte i;
	int mx,my;
	int last_item=0xFF;
	for (i=0;i<MAX_ITEMS_IN_CONTAINER;++i)
	{
		pic = picture_get_from_object(g_containers[container].objects[i]);
		picture_draw(pic,i*20,160-32);
	}
	console_clear();
	console_print(g_containers[container].name,false);
	console_print("\n",false);
	while((mouse_b&2)!=0);
	for(;;)
	{
		mx=mouse_x/4;
		my=mouse_y/2;
		if (mouse_b&2)
		{
			// wait for mouse button release
			while((mouse_b&2)!=0);
			break;
		}
		if (mx/20<MAX_ITEMS_IN_CONTAINER && my>160-32 && my<160)
		{
			obj=g_containers[container].objects[mx/20];
			if (last_item!=mx/20)
			{
				last_item=mx/20;
				if (obj!=OBJECT_NONE)
				{
					console_clear();
					console_print(g_containers[container].name,false);
					console_print("\n",false);
					console_print(g_objects[obj].name,false);
					redraw_console();
				}
				else
				{
					console_clear();
					console_print(g_containers[container].name,false);
					console_print("\n",false);
					console_print("Empty",false);
					redraw_console();
				}
			}
			if (mouse_b&1)
			{
				// wait for mouse button release
				while((mouse_b&1)!=0);
				result=obj;
				break;
			}
		}
	}
	rectfill(screen_buffer,0,(160-32),SCREEN_WIDTH,160,0);
	console_clear();
	redraw_screen();
	return result;
}

object_id ui_object_choose(room_id room)
{
	byte i;
#if SHOW_AREAS
	char *name;
	object_id temp;
	int color;
#endif
	object_id result=OBJECT_NONE;
	object_id area;
	object_id mouse_over_area=OBJECT_NONE;
	object_id last_mouse_over_area=OBJECT_NONE;
	int mx,my;

#if SHOW_AREAS
	for(i=0;i<MAX_AREAS_IN_ROOM;++i)
	{
		area = g_rooms[room].areas[i];
		if (area_activity_check(area))
			color=makecol(0,255,0);
		else
			color=makecol(255,0,0);

		rect(screen,g_areas[area].pos_x*4,g_areas[area].pos_y*2,g_areas[area].pos_x*4+g_areas[area].size_x*4,g_areas[area].pos_y*2+g_areas[area].size_y*2,color);
		temp = object_get_through_references(area);
		if (temp!=OBJECT_NONE)
			textprintf(screen,font,g_areas[area].pos_x*4,g_areas[area].pos_y*2+1,makecol(200,200,200),"%s",g_objects[temp].name);
	}
#endif

	for(;;)
	{
		mx=mouse_x/4;
		my=mouse_y/2;
		mouse_over_area=OBJECT_NONE;
		for(i=0;i<MAX_AREAS_IN_ROOM;++i)
		{
			area = g_rooms[room].areas[i];
			if (area_activity_check(area))
			{
				if (mx>=g_areas[area].pos_x &&
					mx<g_areas[area].pos_x+g_areas[area].size_x &&
					my>=g_areas[area].pos_y &&
					my<g_areas[area].pos_y+g_areas[area].size_y)
				{
					mouse_over_area=area;
					break;
				}
			}
		}
		if (mouse_over_area!=OBJECT_NONE)
		{
			if (last_mouse_over_area!=mouse_over_area)
			{
				last_mouse_over_area=mouse_over_area;
				area=object_get_through_references(mouse_over_area);
				console_clear();
				console_print(g_objects[area].name,false);
				redraw_console();
			}
			if (mouse_b&1)
			{
				result=mouse_over_area;
				break;
			}
		}
		if (mouse_b&2)
		{
			result = ui_container_choose_item(CONTAINER_INVENTORY);
			if (result!=OBJECT_NONE)
				break;
		}
	}
	return object_get_through_references(result);
}

action_function ui_action_choose(object_id obj)
{
	byte i,j;
	int mx,my;
	byte positions[MAX_OBJECT_ACTIONS+1];

	memset(positions,0xFF,sizeof(positions));

	console_clear();
	console_print(g_objects[obj].name,false);
	console_print("\n\n",false); // console_cursor=80
	// 1. print actions
	for (i=0,j=0;i<MAX_OBJECT_ACTIONS;++i)
	{
		if (g_objects[obj].actions[i].name!=TEXT_NONE)
		{
			console_print(g_objects[obj].actions[i].name,false);
			positions[j++]=console_cursor-80;
			console_print("  ",false);
		}
	}
	console_print("Nothing\n",false);
	// 2. Choose action
	for(;;)
	{
		// wait for mouse click
		while((mouse_b&1)!=0);
		while((mouse_b&1)==0);

		mx=mouse_x/4;
		my=mouse_y/2;

		for(i=0;i<MAX_OBJECT_ACTIONS+1;++i)
		{
			if (mx/4<positions[i] && my>=176 && my<184) // /4, because mx is in the gr.15 resolution
			{
				if (positions[i]==0xFF)
				{
					console_clear();
					redraw_console();
					return ACTION_NONE;
				}
				else
				{
					console_clear();
					redraw_console();
					return g_objects[obj].actions[i].action;
				}
				break;
			}
		}
	}
	return ACTION_NONE;
}

//////////////////////////////////////////////////////////////////////////
// Drawing
//////////////////////////////////////////////////////////////////////////

#define PIXEL2RGB(p) (*((rgb*) &p))
#define RGB2PIXEL(p) (*((int*) &p))

#pragma pack(1)

struct rgb {
	unsigned char b;
	unsigned char g;
	unsigned char r;
	unsigned char a;
};

extern byte dli_palette[SCREEN_HEIGHT][NUMBER_OF_COLORS];

struct mic_pixel {
	unsigned char a,b,c,d;
};


struct rgb atari_palette[256];

struct mic_pixel DecodeMicPixel(unsigned char b)
{
	struct mic_pixel p;
	p.a=(b>>6)&3;
	p.b=(b>>4)&3;
	p.c=(b>>2)&3;
	p.d=b&3;
	return p;
}

void picture_draw(picture_id pic, byte pos_x, byte pos_y)
{
	size_t x,y;
	byte pal_id;
	struct mic_pixel p;
	size_t i=0;
	if (pic==PICTURE_NONE)
		return;

	if (g_pictures[pic].data==NULL)
	{
		picture_load(pic);
	}
	pal_id=g_pictures[pic].palette;
	if (pal_id!=PALETTE_NONE)
	{
		palette_set(pal_id,pos_y);
	}

	for (y=0;y<g_pictures[pic].size_y;++y)
	{
		for (x=0;x<g_pictures[pic].size_x/4;++x)
		{
			p = DecodeMicPixel(g_pictures[pic].data[i++]);
			putpixel(screen_buffer,pos_x+x*4+0,pos_y+y,RGB2PIXEL(atari_palette[ dli_palette[y+pos_y][p.a] ]));
			putpixel(screen_buffer,pos_x+x*4+1,pos_y+y,RGB2PIXEL(atari_palette[ dli_palette[y+pos_y][p.b] ]));
			putpixel(screen_buffer,pos_x+x*4+2,pos_y+y,RGB2PIXEL(atari_palette[ dli_palette[y+pos_y][p.c] ]));
			putpixel(screen_buffer,pos_x+x*4+3,pos_y+y,RGB2PIXEL(atari_palette[ dli_palette[y+pos_y][p.d] ]));
		}
	}
	redraw_screen();
}

bool LoadAtariPalette(char *filename)
{
	size_t i;
	struct rgb col;
	FILE *fp;
	col.a=0;

	debug_print("Loading Atari palette:");
	debug_print(filename);

	fp=fopen(filename,"rb");
	if (!fp)
		debug_print("Error loading palette");
	else
	{
		for (i=0;i<256;++i)
		{
			col.r=fgetc(fp);
			col.g=fgetc(fp);
			col.b=fgetc(fp);
			atari_palette[i]=col;
		}
		fclose(fp);
	}
	return true;
}
// Console functions
void console_clear()
{
	clear_bitmap(console_buffer);
	console_cursor=0;
}

void console_wait_for_fire()
{
	set_mouse_sprite(mouse_sprite_waiting);

	// wait for mouse button released
	while((mouse_b&1)!=0);
	// wait for mouse button pressed
	while((mouse_b&1)==0);
	// wait for mouse button released
	while((mouse_b&1)!=0);

	set_mouse_sprite(mouse_sprite_normal);
}

void console_print(unsigned char *s, bool wait)
{
	size_t x,y;
	char temp[2];
	temp[1]=0;
	while(*s!=0)
	{
		if (*s=='\n')
		{
			console_cursor+=40-console_cursor%40;
			++s;
		}
		else
		{
			temp[0]=*s++;
			x=console_cursor%40;
			y=console_cursor/40;
			textout_ex(console_buffer,font,temp,x*8,y*8,makecol(255,255,255),0);
			++console_cursor;
		}
		if (console_cursor>=40*4)
		{
			redraw_console();
			console_wait_for_fire();
			console_clear();
			console_cursor=0;
		}
	}
	redraw_console();
	if (wait)
	{
		console_wait_for_fire();
		console_clear();
		redraw_console();
	}
}



void system_init()
{
	allegro_init(); // Initialize Allegro
	install_mouse();
	install_keyboard();
	set_color_depth(32);
	set_gfx_mode(GFX_AUTODETECT_WINDOWED, 800,600,0,0); // Change our graphics mode to 640x480
	set_display_switch_mode(SWITCH_BACKGROUND);
	mouse_sprite_normal = load_bitmap("debug\\cursor1.bmp",NULL);
	mouse_sprite_waiting = load_bitmap("debug\\cursor2.bmp",NULL);
	set_mouse_sprite(mouse_sprite_normal);
	show_mouse(screen);
	LoadAtariPalette("laoo.act");
	debug_print("Allegro library initialized.");
	screen_buffer=create_bitmap_ex(32,SCREEN_WIDTH,SCREEN_HEIGHT);
	clear_bitmap(screen_buffer);

	console_buffer=create_bitmap_ex(32,40*8,4*8);
	console_clear();
	// Set the border
	line(screen,0,SCREEN_HEIGHT*2,SCREEN_WIDTH*4,SCREEN_HEIGHT*2,makecol(30,30,30));
	line(screen,SCREEN_WIDTH*4,0,SCREEN_WIDTH*4,SCREEN_HEIGHT*2,makecol(30,30,30));
}


#endif


