#ifdef __ATARI__

#include <atari.h> 
#include <stdio.h>
#include <peekpoke.h>
#include <joystick.h>
#include <stdlib.h> 
#include <string.h>
#include "game\all_definitions.h"
#include "engine\atari_fn.h"

//////////////////////////////////////////////////////////////////////////
// Addresses of data under ROM
//////////////////////////////////////////////////////////////////////////
// 1024 bytes
#define sprites ((byte *)(0xd800))

// 4000 bytes
#define dli_handler_address 0xF000
#define dli_handler ((byte *)(dli_handler_address))
//byte dli_handler[4000];

//////////////////////////////////////////////////////////////////////////
#define MEM_MANAGER_FIFO_SIZE 10
byte *mem_manager_fifo[MEM_MANAGER_FIFO_SIZE+1];
byte mem_manager_allocated=0;
//////////////////////////////////////////////////////////////////////////
extern byte dli_palette[SCREEN_HEIGHT][NUMBER_OF_COLORS];
extern struct s_object g_objects[OBJECT_LAST_OBJECT];
extern struct s_picture g_pictures[PICTURE_LAST_PICTURE];
extern struct s_palette g_palettes[PALETTE_LAST_PALETTE];
extern struct s_room g_rooms[ROOM_LAST_ROOM];
extern struct s_area g_areas[AREA_LAST_AREA];
extern struct s_container g_containers[CONTAINER_LAST_CONTAINER];
//////////////////////////////////////////////////////////////////////////

void *memory_manager_alloc(word size)
{
	byte *result;
	byte i;
	for(;;)
	{
		if (mem_manager_allocated<MEM_MANAGER_FIFO_SIZE)
		{
			result=malloc(size);
			if (result!=0)
			{
				mem_manager_fifo[mem_manager_allocated++]=result;
				return result;
			}
		}
		
		result=mem_manager_fifo[0];
		if (result==0)
			return 0;

		// check pictures
		for (i=0;i<PICTURE_LAST_PICTURE;++i)
		{
			if (g_pictures[i].data==result)
				g_pictures[i].data=NULL;
		}
		// check palettes
		for (i=0;i<PALETTE_LAST_PALETTE;++i)
		{
			if (g_palettes[i].data==result)
				g_palettes[i].data=NULL;
		}

		// remove the first in fifo
		free(result);
		--mem_manager_allocated;
		for (i=0;i<MEM_MANAGER_FIFO_SIZE-1;++i)
		{
			mem_manager_fifo[i]=mem_manager_fifo[i+1];
		}
	};
}

void rom_off();
void rom_on();
void play_sfx(byte effect);
void play_music(byte track_number);
void stop_music();

byte joy;
unsigned char *video_ptr;
unsigned screen_memory;
char *console_memory;

extern char joy_driver;

// cursor position
byte cursor_x, cursor_y;
byte cursor_data[8]={0,0xC0,0xE0,0xF0,0xF8,0xFC,0xE0,0};
byte cursor_data2[6]={0,0x80,0xC0,0xE0,0x80,0};

void redraw_cursor()
{
	register byte *ptr;
	ptr=sprites+0x200 + 16; // 0x200 - first sprite + screen top
	ptr+=cursor_y/2;
	POKE(HPOSP1,cursor_x+48); 
	POKE(HPOSP0,cursor_x+49); 
	memcpy(ptr,cursor_data2,6);
	memcpy(ptr+0x7f,cursor_data,8); // second sprite
}

void move_cursor_up(void)
{
	if (cursor_y>1)
	{
		cursor_y-=2;
		redraw_cursor();
	}
}

void move_cursor_down(void)
{
	if (cursor_y<MAIN_PICTURE_HEIGHT-10)
	{
		cursor_y+=2;
		redraw_cursor();
	}
}

void move_cursor_left(void)
{
	if (cursor_x>0)
	{
		--cursor_x;
		redraw_cursor();
	}
}

void move_cursor_right(void)
{
	if (cursor_x<MAIN_PICTURE_WIDTH-1)
	{
		++cursor_x;
		redraw_cursor();
	}
}

void wait(unsigned char t)
{
	POKE(540,t);
	while(PEEK(540));
}

bool fire_released(void)
{
	if (JOY_BTN_FIRE(joy_read(JOY_1)))
	{
		while(JOY_BTN_FIRE(joy_read(JOY_1)));
		return true;
	}
	return false;
}

void console_wait_for_fire(void)
{
	do 
	{
		joy=joy_read(JOY_1);
	} while (!JOY_BTN_FIRE(joy));
	do 
	{
		POKE(0x4D,0); // Disable Attract mode
		joy=joy_read(JOY_1);
	} while (JOY_BTN_FIRE(joy));	
}

byte console_cursor=0;

void console_clear(void)
{
	register byte i;
	for (i=0;i<160;++i)
		console_memory[i]=0;
//	hm... doesn't work???? - memset(console_memory,0,160);
	console_cursor=0;
}


void console_print_inverted(unsigned char *s)
{
	byte c;
	while(*s!=0)
	{
		c=*s++;
		if (c==0)
			break;
		else if (c<32)
			c+=64;
		else if (c<96)
			c-=32;
		console_memory[console_cursor++]=c+128;
	}
}

void palette_generate_dli(void)
{
	register byte *dli_data=dli_handler;
	register byte y,i,reg;

	// pha
	*dli_data++=0x48;

	for(y=0;y<MAIN_PICTURE_HEIGHT;++y)
	{
		// wsync
		*dli_data++=0x8D;
		*dli_data++=0x0A;
		*dli_data++=0xD4;

		for (i=0;i<4;++i)
		{
			switch(i)
			{
			case 0:
				reg=0x1A;
				break;
			case 1:
				reg=0x16;
				break;
			case 2:
				reg=0x17;
				break;
			case 3:
				reg=0x18;
				break;
			}
			*dli_data++=0xA9; // LDA
			*dli_data++=dli_palette[y][i]; // value
			*dli_data++=0x8D; // STA
			*dli_data++=reg; // reg
			*dli_data++=0xD0; // reg
		}
	}
	// wsync
	*dli_data++=0x8D;
	*dli_data++=0x0A;
	*dli_data++=0xD4;
	// store black border color at the end
	*dli_data++=0xA9; // LDA
	*dli_data++=0x0; // 0
	*dli_data++=0x8D; // STA
	*dli_data++=0x18; // border
	*dli_data++=0xD0; // reg

	// store font color
	*dli_data++=0xA9; // LDA
	*dli_data++=0x0F; // white
	*dli_data++=0x8D; // STA
	*dli_data++=0x17; // font color
	*dli_data++=0xD0; // reg

	// pla
	*dli_data++=0x68;

	*dli_data=0x40; // RTI
}


void picture_draw(picture_id pic, byte pos_x, byte pos_y)
{
	register byte y;
	register word i=0;
	register byte *ptr;
	byte line_length;
	byte pal_id;

	if (pic==PICTURE_NONE)
		return;

	if (g_pictures[pic].data==NULL)
	{
		picture_load(pic);
	}
	pal_id=g_pictures[pic].palette;
	if (pal_id!=PALETTE_NONE)
	{
		palette_set(pal_id,pos_y);
		palette_generate_dli();
	}

	line_length=g_pictures[pic].size_x/4;
	ptr=video_ptr+pos_x/4+pos_y*40;
	for (y=0;y<g_pictures[pic].size_y;++y)
	{
		memcpy(ptr,g_pictures[pic].data+i,line_length);
		ptr+=40;
		i+=line_length;
	}
}

void console_print(unsigned char *s, bool wait)
{
	byte c;
	while(*s!=0)
	{
		if (*s=='\n')
		{
			console_cursor+=40-console_cursor%40;
			++s;
		}
		else
		{
			c=*s++;
			if (c==0)
				break;
			else if (c<32)
				c+=64;
			else if (c<96)
				c-=32;
			console_memory[console_cursor++]=c;
		}
		if (console_cursor>=40*4)
		{
			console_wait_for_fire();
			console_clear();
		}
	}

	if (wait)
	{
		console_wait_for_fire();
		console_clear();
	}
}


// display container elements on the screen
// can be as gfx or as text
// In this demo we show as graphics
object_id ui_container_choose_item(container_id container)
{
	byte i,last=0xFF;
	object_id result=OBJECT_NONE;
	for (i=0;i<MAX_ITEMS_IN_CONTAINER;++i)
	{
		picture_draw(picture_get_from_object(g_containers[container].objects[i]),i*20,160-32);
	}
	i=cursor_x/20;
	cursor_y=160-32;
	memset(sprites,0,1024);
	if (i>MAX_ITEMS_IN_CONTAINER-1)
		i=MAX_ITEMS_IN_CONTAINER-1;
	// choose item with fire
	for(;;)
	{
		joy=joy_read(JOY_1);
		if (JOY_BTN_LEFT(joy))
		{
			if (i>0)
				--i;
		}
		else if (JOY_BTN_RIGHT(joy))
		{
			if (i<MAX_ITEMS_IN_CONTAINER-1)
				++i;
		}
		else if (JOY_BTN_UP(joy))
		{
			result=OBJECT_NONE;
			break;
		}
		result=g_containers[container].objects[i];
		cursor_x=i*20;
		redraw_cursor();
		if (fire_released())
		{
			break;
		}
		if (i!=last)
		{
			last=i;
			console_clear();
			console_print(g_containers[container].name,false);
			console_print("\n",false);
			if (result!=OBJECT_NONE)
				console_print(g_objects[result].name,false);
			else
				console_print("Empty",false);
		}
		wait(5);
	}
	// hide inventory by changing the palette
	memset(&dli_palette[160-32][0],0,4*32);
	palette_generate_dli();
	console_clear();
	return result;
}

void move_cursor(void)
{
	joy=joy_read(JOY_1);

	if (JOY_BTN_LEFT(joy))
	{
		move_cursor_left();			
	}
	else if (JOY_BTN_RIGHT(joy))
	{
		move_cursor_right();
	}

	if (JOY_BTN_UP(joy))
	{
		move_cursor_up();
	}
	else if (JOY_BTN_DOWN(joy))
	{
		move_cursor_down();
	}
	wait(1);
}

// Two types of choosing
// The first one - user moves the cursor on the screen
// The second one - the cursor jumps from area to area
object_id ui_object_choose(room_id room)
{
	register byte i;
	object_id result=OBJECT_NONE;
	object_id cursor_over_area;
	object_id last_cursor_over_area;
	object_id area;
	for(;;)
	{
		move_cursor();
		cursor_over_area=OBJECT_NONE;
		for(i=0;i<MAX_AREAS_IN_ROOM;++i)
		{
			area = g_rooms[room].areas[i];
			if (area_activity_check(area))
			{
				if (cursor_x>=g_areas[area].pos_x &&
					cursor_x<g_areas[area].pos_x+g_areas[area].size_x &&
					cursor_y>=g_areas[area].pos_y &&
					cursor_y<g_areas[area].pos_y+g_areas[area].size_y)
				{
					cursor_over_area=area;
					break;
				}
			}
		}
		if (cursor_over_area!=OBJECT_NONE)
		{
			if (last_cursor_over_area!=cursor_over_area)
			{
				last_cursor_over_area=cursor_over_area;
				area=object_get_through_references(cursor_over_area);
				console_clear();
				console_print(g_objects[area].name,false);
			}
			if (fire_released())
			{
				result=cursor_over_area;
				break;
			}
		}
		if (cursor_y==MAIN_PICTURE_HEIGHT-10)
		{
			cursor_y-=2;
			redraw_cursor();
			console_clear();
			console_print("Inventory",false);
			result = ui_container_choose_item(CONTAINER_INVENTORY);
			if (result!=OBJECT_NONE)
				break;
		}
	}
	return object_get_through_references(result);
}

action_function ui_action_choose(object_id obj)
{
	byte i;
	byte *name;
	byte current=0;

	console_clear();
	console_print(g_objects[obj].name,false);
	for(;;)
	{
		joy=joy_read(JOY_1);
		if (JOY_BTN_LEFT(joy))
		{
			if (current>0)
			{
				--current;
				while(current>0 && g_objects[obj].actions[current].name==TEXT_NONE)
					--current;
			}
		}
		else if (JOY_BTN_RIGHT(joy))
		{
			if (current<MAX_OBJECT_ACTIONS)
			{
				++current;
				while(current<MAX_OBJECT_ACTIONS && g_objects[obj].actions[current].name==TEXT_NONE)
					++current;
			}
		}
		console_cursor=80;
		for (i=0;i<MAX_OBJECT_ACTIONS;++i)
		{
			name=g_objects[obj].actions[i].name;
			if (name!=TEXT_NONE)
			{
				if (current==i)
					console_print_inverted(name);
				else
					console_print(name,false);
			}
			console_cursor+=2;
		}
		if (current==i)
		{
			console_print_inverted("Nothing");
			if (fire_released())
			{
				console_clear();
				return ACTION_NONE;
			}
		}
		else
		{
			console_print("Nothing",false);
			if (fire_released())
			{
				console_clear();
				return g_objects[obj].actions[current].action;
			}
		}
		wait(5);
	}
}

void cursor_init(void)
{

	POKE(SDMCTL,DMACTL_ENABLE_PLAYER_DMA|DMACTL_ENABLE_MISSLE_DMA|DMACTL_NORMAL_PLAYFIELD|DMACTL_DMA_FETCH_INSTRUCTION); 
	POKE(PMBASE,((word) sprites)/256);
	memset(sprites,0,1024);
	POKE(GRACTL,3); // turn on PMG

	POKE(GPRIOR,1); // set non-highlighting sprite
	POKE(PCOLR0,0x0F); // set color
	POKE(PCOLR1,0x00); // set color

	POKE(SIZEP0,0);
	POKE(SIZEP1,0);

	cursor_y=2;
	cursor_x=0;
	redraw_cursor();
}

void init_palette()
{
	register byte i;
	for (i=0;i<SCREEN_HEIGHT;++i)
	{
		dli_palette[i][0]=0;
		dli_palette[i][1]=0x4;
		dli_palette[i][2]=0xa;
		dli_palette[i][3]=0xf;
	}
	palette_generate_dli();

	// Set dli handler
	POKEW(0x200,(word) dli_handler);

	// Enable DLI
	POKE(0xD40E,0xC0);
}

void system_init(void)
{
	rom_off();
	// set KEYBD vector to RTI, because by default it points to our memory under ROM (FC19)
	POKEW(0x208,0xC0CD);	

	_graphics(15); 

	joy_install(&joy_driver);
	
	screen_memory=PEEKW(560)+4;
	video_ptr=(unsigned char*)(PEEKW(screen_memory));
	console_memory=(byte *) 0xBF60; // predefined
	cursor_init();
	init_palette();
}


#endif
