#ifndef OBJECT_H
#define OBJECT_H

#include "engine\config.h"
#include "engine\action.h"
#include "engine\area.h"
#include "engine\basictypes.h"

#define OBJECT_NONE 0xFF
typedef byte object_id;

struct s_context_command {
	char *name;
	action_function action;
};

struct s_object {
	// if name==TEXT_NONE then object is reference to other object, 
	// for example OBJECT_HALL_FIRST_DOOR is reference to OBJECT_CELL_DOOR
	// because they share the same object
	// the reference number is in variable
	// This way you can operate on the same door from two different rooms
	char *name; 

	// this is an universal data variable for object
	// f.e. for Cell Door it stores state as flags (CLOSED|LOCKED) 
	// for area it stores information about activity
	byte variable; 
	struct s_context_command actions[MAX_OBJECT_ACTIONS];
};

#define ACTIVITY_MASK 0x80
#define AREA_ACTIVE 0
#define AREA_INACTIVE ACTIVITY_MASK
#define VALUE_MASK 0x7F

//////////////////////////////////////////////////////////////////////////
// O
//////////////////////////////////////////////////////////////////////////

// Functions to access data stored in objects
// Use always them instead of direct access, because they:
// 1. encode/decode bits for area/objects
// not implemented: 2. pass over object references
byte object_value_get(object_id obj);
void object_value_set(object_id obj, byte value);
bool object_flags_check  (object_id obj,  byte flags);
void object_flags_set  (object_id obj,  byte flags);
void object_flags_zero (object_id obj,  byte flags);

void object_action_replace(object_id obj, action_function old_function, action_function new_function, char *new_name);
char *object_name_get(object_id obj);
void object_name_set(object_id obj, char *new_name);

object_id object_get_through_references(object_id obj);

void area_activity_on(object_id ar);
void area_activity_off(object_id ar);
bool area_activity_check(object_id ar);

#endif
