#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "game\all_definitions.h"
#include "engine\memmanager.h"

#ifdef __ATARI__
#include <peekpoke.h>
#include "engine\atari_fn.h"
void wait(unsigned char t);
#endif

// The palette on the whole screen
byte dli_palette[SCREEN_HEIGHT][NUMBER_OF_COLORS];

extern struct s_picture g_pictures[PICTURE_LAST_PICTURE];
extern struct s_palette g_palettes[PALETTE_LAST_PALETTE];
extern struct s_picture_object_mapping g_picture_object_mapping[MAX_PICTURE_OBJECT_MAPPINGS];

picture_id picture_get_from_object(object_id obj)
{
	byte i;
	for (i=0;i<MAX_PICTURE_OBJECT_MAPPINGS;++i)
	{
		if (g_picture_object_mapping[i].object==obj)
			return g_picture_object_mapping[i].picture;
	}
	return PICTURE_NONE;
}

void palette_set(palette_id pal, byte pos_y)
{
	if (pal==PALETTE_NONE)
		return;

	if (g_palettes[pal].data==NULL)
		palette_load(pal);
	memcpy(&dli_palette[pos_y][0],g_palettes[pal].data,g_palettes[pal].data_size);
}

bool picture_load(picture_id pic)
{
	FILE *fp;
	size_t data_size=g_pictures[pic].size_x/4;
	data_size*=g_pictures[pic].size_y;

#ifdef __ATARI__
	// disable ANTIC (screen)
	POKE(SDMCTL,0);
	// disable DLI
	POKE(0xD40E,0x40);
	POKE(0xA052,0x70);
	wait(1);
#endif

	g_pictures[pic].data=(byte *) memory_manager_alloc(data_size);
	if (g_pictures[pic].data==NULL)
	{
//		console_print("No memory for the picture",true);
		return false;
	}

	fp=fopen(g_pictures[pic].source,"rb");
	if (!fp)
	{
//		console_print("Can't load the picture",true);
		return false;
	}
	fread(g_pictures[pic].data,1,data_size,fp);
	fclose(fp);

#ifdef __ATARI__
	// Enable DLI
	POKE(0xD40E,0xC0);
	POKE(0xA052,0xF0);
	// enable screen
	// !!! WARNING - in case of error the screen stays disabled!
	POKE(SDMCTL,DMACTL_ENABLE_PLAYER_DMA|DMACTL_ENABLE_MISSLE_DMA|DMACTL_NORMAL_PLAYFIELD|DMACTL_DMA_FETCH_INSTRUCTION); 
#endif
	return true;
}

bool palette_load(palette_id pal)
{
	FILE *fp;

#ifdef __ATARI__
	// disable ANTIC (screen)
	POKE(SDMCTL,0);
	// disable DLI
	POKE(0xD40E,0x40);
	POKE(0xA052,0x70);
	wait(1);
#endif

	g_palettes[pal].data=(byte *) memory_manager_alloc(g_palettes[pal].data_size);
	if (g_palettes[pal].data==NULL)
	{
//		console_print("No memory for the palette",true);
		return false;
	}
	fp=fopen(g_palettes[pal].source,"rb");
	if (!fp)
	{
//		console_print("Can't load the palette",true);
		return false;
	}
	fread(g_palettes[pal].data,1,g_palettes[pal].data_size,fp);
	fclose(fp);

#ifdef __ATARI__
	// Enable DLI
	POKE(0xD40E,0xC0);
	POKE(0xA052,0xF0);
	// enable screen
	// !!! WARNING - in case of error the screen stays disabled!
	POKE(SDMCTL,DMACTL_ENABLE_PLAYER_DMA|DMACTL_ENABLE_MISSLE_DMA|DMACTL_NORMAL_PLAYFIELD|DMACTL_DMA_FETCH_INSTRUCTION); 
#endif
	return true;
}