#ifndef PICTURE_H
#define PICTURE_H

#include "engine\basictypes.h"
#include "engine\config.h"
#include "engine\object.h"

#define PICTURE_NONE 0xFF
#define PALETTE_NONE 0xFF

typedef byte picture_id;
typedef byte palette_id;

struct s_picture {
	// source may be a file name or pointer to memory, if it's stored there
	byte *source;
	byte size_x;
	byte size_y;
	palette_id palette;
	byte *data;
};

struct s_palette {
	// source may be a file name or pointer to memory, if it's stored there
	byte *source;
	word data_size;
	byte *data;
};

struct s_picture_object_mapping {
	object_id object;
	picture_id picture;
};

//////////////////////////////////////////////////////////////////////////
// Picture API
//////////////////////////////////////////////////////////////////////////

void screen_clear();

// draw picture on screen 
void picture_draw(picture_id pic, byte pos_x, byte pos_y);

// by user defined mapping
picture_id picture_get_from_object(object_id obj);

// Loads picture from disk, memory or from any other storage
// Depending on the implementation it may decompress it or not
// As a result 'data' pointer is set
// You may preload a few pictures to avoid loading breaks, if you have free memory
bool picture_load(picture_id pic);

// not implemented - void picture_free(picture_id pic);

//////////////////////////////////////////////////////////////////////////
// Palette API
//////////////////////////////////////////////////////////////////////////

// Loads palette from disk, memory or from any other storage
// Depending on the implementation it may decompress it or not
// As a result 'data' pointer is set
bool palette_load(palette_id);

// generates screen palette on 
void palette_set(palette_id pal, byte pos_y);

// sets all the colors to 0 - to hide the picture, useful during the room change and drawing room changes
void palette_zero(byte start_y, byte end_y);

// generates screen DLI by the current palette 
// this changes colors on the screen
void palette_generate_dli(void);

#endif
