#pragma warning (disable: 4312)
#pragma warning (disable: 4996)
#pragma warning (disable: 4267)

#include <math.h>
#include <vector>
#include <map>
#include <set>
#include <algorithm>
#include <string>
#include <allegro.h> 
#include "string_conv.h"

using namespace std;

#define PIXEL2RGB(p) (*((rgb*) &p))
#define RGB2PIXEL(p) (*((int*) &p))

void error(const char *e)
{
	allegro_message(e);
	allegro_exit();
	exit(1);
}

void error(const char *format,const char *e)
{
	allegro_message(format,e);
	allegro_exit();
	exit(1);
}


#pragma pack(1)

struct rgb {
	unsigned char b;
	unsigned char g;
	unsigned char r;
	unsigned char a;
	bool operator==(const rgb &ar)
	{
		if (r==ar.r && g==ar.g && b==ar.b)
			return true;
		else
			return false;
	}
};

struct mic_pixel {
	unsigned char a,b,c,d;
};

vector <mic_pixel> mic_line;

typedef vector < unsigned char > file_buffer;

vector < rgb > atari_palette; // 256 colors 

vector < vector < unsigned char > > dli_colors;

//////////////////////////////////////////////////////////////////////////
// req arguments
char *mic_input;
char *col_input;
size_t pic_size_x;
size_t pic_size_y;
//////////////////////////////////////////////////////////////////////////
// for automatic
size_t cut_x;
size_t cut_y;
size_t cut_w;
size_t cut_h;

string mic_output;
string dli_output;
//////////////////////////////////////////////////////////////////////////

// global data
file_buffer input_buffer;
BITMAP *bitmap=0;

//////////////////////////////////////////////////////////////////////////

bool LoadAtariPalette(const char *filename)
{
	size_t i;
	rgb col;
	col.a=0;
	FILE *fp=fopen(filename,"rb");
	if (!fp)
		error("Error loading palette %s",filename);
	for (i=0;i<256;++i)
	{
		col.r=fgetc(fp);
		col.g=fgetc(fp);
		col.b=fgetc(fp);
		atari_palette.push_back(col);
	}
	fclose(fp);
	return true;
}

file_buffer ReadFileBuffer(char *filename)
{
	FILE *fp=fopen(filename,"rb");
	file_buffer result;
	if (!fp)
		error("Error loading file %s", filename);	

	unsigned char loaded;
	while(fread(&loaded,1,1,fp)==1)
		result.push_back(loaded);
	fclose(fp);
	return result;
}

void SaveDLIFile(const char *filename, size_t pos_y,size_t size_y)
{
	FILE *fp=fopen(filename,"wb+");
	if (!fp)
		error("Error saving file %s", filename);	

	size_t i,y;
	for (y=pos_y;y<pos_y+size_y;++y)
	{
		for (i=0;i<4;++i)
		{
			fputc(dli_colors[y][i],fp);
		}
	}
	fclose(fp);
}

void SaveFileBuffer(const char *filename,file_buffer &to_save)
{
	FILE *fp=fopen(filename,"wb+");
	if (!fp)
		error("Error saving file %s", filename);	

	size_t i;
	for (i=0;i<to_save.size();++i)
		fputc(to_save[i],fp);
	fclose(fp);
}

mic_pixel DecodeMicPixel(unsigned char b)
{
	mic_pixel p;
	p.a=(b>>6)&3;
	p.b=(b>>4)&3;
	p.c=(b>>2)&3;
	p.d=b&3;
	return p;
}

void ShowMicOnBitmap(BITMAP *bmp,const file_buffer &buffer,size_t size_x, size_t size_y, size_t dli_line, size_t pos_x,size_t pos_y)
{
	size_t x,y;
	size_t i=0;
	BITMAP *temp=create_bitmap_ex(32,bmp->w,bmp->h);
	for (y=0;y<size_y;++y)
	{
		for (x=0;x<size_x;++x)
		{
			mic_pixel p = DecodeMicPixel(buffer[i++]);
			putpixel(temp,pos_x+x*8+0,pos_y+y,RGB2PIXEL(atari_palette[ dli_colors[y+dli_line][p.a] ]));
			putpixel(temp,pos_x+x*8+1,pos_y+y,RGB2PIXEL(atari_palette[ dli_colors[y+dli_line][p.a] ]));
			putpixel(temp,pos_x+x*8+2,pos_y+y,RGB2PIXEL(atari_palette[ dli_colors[y+dli_line][p.b] ]));
			putpixel(temp,pos_x+x*8+3,pos_y+y,RGB2PIXEL(atari_palette[ dli_colors[y+dli_line][p.b] ]));
			putpixel(temp,pos_x+x*8+4,pos_y+y,RGB2PIXEL(atari_palette[ dli_colors[y+dli_line][p.c] ]));
			putpixel(temp,pos_x+x*8+5,pos_y+y,RGB2PIXEL(atari_palette[ dli_colors[y+dli_line][p.c] ]));
			putpixel(temp,pos_x+x*8+6,pos_y+y,RGB2PIXEL(atari_palette[ dli_colors[y+dli_line][p.d] ]));
			putpixel(temp,pos_x+x*8+7,pos_y+y,RGB2PIXEL(atari_palette[ dli_colors[y+dli_line][p.d] ]));
		}
	}
	blit(temp,bmp,0,0,0,0,bmp->w,bmp->h);
	destroy_bitmap(temp);
}

void LoadCOLFile(char *filename)
{
	size_t y;

	FILE *fp=fopen(filename,"rb");
	if (!fp)
		error("Error opening COL file");

	dli_colors.resize(pic_size_y);

	for (size_t i=0;i<4;++i)
	{
		for(y=0;y<256;++y)
		{
			unsigned char val=fgetc(fp);
			if (feof(fp))
			{
				val=i;
			}
			if (y<pic_size_y)
			{
				dli_colors[y].push_back(val);
			}
		}
	}
	fclose(fp);
}

void Automatic()
{
	cut_x/=4;
	cut_w/=4;
	if (cut_w==0)
		++cut_w;

	ShowMicOnBitmap(screen,input_buffer,pic_size_x,pic_size_y,0,0,0);

	// 2. Wyciecie MIC (cut_x=cut_x/4)

	file_buffer output_buffer(cut_w*cut_h);

	size_t x,y;
	size_t out_x, out_y;
	for (y=cut_y,out_y=0;y<cut_y+cut_h;++y,++out_y)
	{

		for (x=cut_x,out_x=0;x<cut_x+cut_w;++x,++out_x)
		{
			output_buffer[out_x+cut_w*out_y]=input_buffer[x+pic_size_x*y];
		}
	}
	SaveFileBuffer(mic_output.c_str(),output_buffer);

	ShowMicOnBitmap(screen,output_buffer,cut_w,cut_h,cut_y,0,300);
	SaveDLIFile(dli_output.c_str(),cut_y,cut_h);

	BITMAP *temp=create_bitmap_ex(32,cut_w*8,cut_h);
	ShowMicOnBitmap(temp,output_buffer,cut_w,cut_h,cut_y,0,0);
	PALETTE pal;
	get_palette(pal);
	save_bmp(string(mic_output.substr(0,mic_output.size()-4)+".bmp").c_str(),temp,pal);
	destroy_bitmap(temp);
}

size_t grid_x=0, grid_y=0;
size_t mx,my;
size_t grid_nx=0, grid_ny=0;
int ax1=0;
int ay1=0;
int ax2=0;
int ay2=0;

void SaveArea()
{
	cut_x=ax1/2;
	cut_y=ay1;
	cut_w=(ax2-ax1)/2;
	cut_h=ay2-ay1;

	mic_output=mic_input;

	string info;
	info+=Value2String<int>(ax1/2);
	info+=",";
	info+=Value2String<int>(ay1);
	info+="(";
	info+=Value2String<int>(cut_w);
	info+=",";
	info+=Value2String<int>(cut_h);
	info+=")";

	mic_output+=info;
	mic_output+=".mic";

	dli_output=mic_input;
	dli_output+=info;
	dli_output+=".dli";
	Automatic();
}


void Interactive()
{
	bitmap = create_bitmap_ex(32,pic_size_x*4*2,pic_size_y);
	ShowMicOnBitmap(screen,input_buffer,pic_size_x,pic_size_y,0,0,0);
	blit(screen,bitmap,0,0,0,0,pic_size_x*8,pic_size_y);

	for (;;)
	{
		mx=mouse_x;
		my=mouse_y;

		if (key[KEY_ESC])
			break;

		mx=min(mx,pic_size_x*8-8);
		my=min(my,pic_size_y-1);
		if (!(grid_nx==grid_x && grid_ny==grid_y))
		{
			blit(bitmap,screen,0,0,0,0,pic_size_x*8,pic_size_y);
			rect(screen,ax1-2,ay1-2,ax2+2,ay2+2,makecol(0,0,0));
			rect(screen,ax1-1,ay1-1,ax2+1,ay2+1,makecol(255,255,255));

			grid_ny=grid_y;
			grid_nx=grid_x;

			size_t sx=pic_size_x*8-grid_nx;
			if (sx>50)
				sx=50;
			size_t sy=pic_size_y-grid_ny;
			if (sy>50)
				sy=50;
			if (sx!=0 && sy!=0)
			{
				blit(bitmap,screen,0,0,0,0,pic_size_x*8,pic_size_y);
				stretch_blit(bitmap,screen,grid_nx,grid_ny,sx,sy,400,0,300,300);
			}
			line(screen,0,grid_y-1,pic_size_x*8-1,grid_y-1,makecol(255,255,255));
			line(screen,0,grid_y-2,pic_size_x*8-1,grid_y-2,makecol(0,0,0));
			line(screen,grid_x-1,0,grid_x-1,pic_size_y-1,makecol(255,255,255));
			line(screen,grid_x-2,0,grid_x-2,pic_size_y-1,makecol(0,0,0));
		}
		grid_x=mx-(mx%8);
		grid_y=my;
		textprintf_ex(screen,font,0,pic_size_y+10,makecol(255,255,0),0,"%d,%d         ",grid_x/2,grid_y);
		while(mouse_b&1)
		{
			mx=mouse_x;
			my=mouse_y;

			grid_nx=min(mx-(mx%8),pic_size_x*8);
			grid_ny=min(my,pic_size_y);
			if (grid_nx>grid_x && grid_ny>grid_y)
			{
				blit(bitmap,screen,0,0,0,0,pic_size_x*8,pic_size_y);
				stretch_blit(bitmap,screen,grid_x,grid_y,grid_nx-grid_x,grid_ny-grid_y,400,0,300,300);
				rect(screen,grid_x-2,grid_y-2,grid_nx+2,grid_ny+2,makecol(0,0,0));
				rect(screen,grid_x-1,grid_y-1,grid_nx+1,grid_ny+1,makecol(255,255,255));
			}					
			ax1=grid_x;
			ay1=grid_y;
			ax2=grid_nx;
			ay2=grid_ny;
			textprintf_ex(screen,font,0,pic_size_y+20,makecol(255,255,0),0,"%d,%d-%d,%d (%d x %d)         ",ax1/2,ay1,ax2/2,ay2,(ax2-ax1)/2,ay2-ay1);
		}
		rect(screen,ax1-2,ay1-2,ax2+2,ay2+2,makecol(0,0,0));
		rect(screen,ax1-1,ay1-1,ax2+1,ay2+1,makecol(255,255,255));

		if (mouse_b&2)
		{
			SaveArea();
			grid_nx=-1; // show bitmap 
			textprintf_ex(screen,font,0,pic_size_y+20,makecol(255,255,0),0,"%d,%d-%d,%d (%d x %d)         ",ax1/2,ay1,ax2/2,ay2,(ax2-ax1)/2,ay2-ay1);
		}
	}
}


int main(int argc, char *argv[])
{
//	CommandLineParser parser; 
//	parser.parse(argc, argv);

	allegro_init(); // Initialize Allegro
	install_mouse();
	install_keyboard();
	set_color_depth(32);
	set_gfx_mode(GFX_AUTODETECT_WINDOWED, 1000,600,0,0); // Change our graphics mode to 640x480
//	set_gfx_mode(GFX_AUTODETECT_WINDOWED, 1024,768,0,0); // Change our graphics mode to 640x480
	set_display_switch_mode(SWITCH_BACKGROUND);
	show_mouse(screen);

	bool interactive=true;

	if (argc>10)
	{
		cut_x= String2Value <size_t> (string(argv[5]));
		cut_y= String2Value <size_t> (string(argv[6]));
		cut_w= String2Value <size_t> (string(argv[7]));
		cut_h= String2Value <size_t> (string(argv[8]));
		mic_output=argv[9];
		dli_output=argv[10];
		interactive=false;
	}

	if (argc>4)
	{
		mic_input=argv[1];
		col_input=argv[2];
		pic_size_x= String2Value <size_t> (string(argv[3]));
		pic_size_y= String2Value <size_t> (string(argv[4]));;
	}
	else 
		error("slicer img.mic img.col size_x size_y [cut_x cut_y cut_w cut_h out.mic out.dli]");

	LoadAtariPalette("laoo.act");

	// slicer img.mic img.col sx sx x y w h out.mic out.dli
	
	//////////////////////////////////////////////////////////////////////////
	//  zakodowany do pamici (4 piksele/bajt)
	//  wycinamy po pelnych bajtach
	pic_size_x/=4;
	input_buffer = ReadFileBuffer(mic_input);
	LoadCOLFile(col_input);

	if (interactive)
		Interactive();
	else
		Automatic();
	return 0;
}


END_OF_MAIN() // This must be called right after the closing bracket of your MAIN function.
// It is Allegro specific.